#!/usr/bin/perl -w

#### LAST UPDATED: Jun, 2003 ####
# learnPOS.pl version 0.1

###########################################################################
#
# This program learns POS features from a given training file, SOURCE.
# The user specifies the positions of the features he is interested in,
# relative to the target word. L1 stands for the pos feature one position
# to the left of the target word. Ln  stands for the pos feature n positions
# to the left. Similary, R1, R2, ... , Rn. The program captures the pos
# occurring at the positions specified and the number of times they occur
# at that position. Those parts of speech which occur more than a user 
# specified number of times (MIN), along with the corresponding
# position constitute a learned feature. This feature, position and pos,
# are then printed out to the OUTPUT file. Each entry in OUTPUT corresponds
# to one feature. The position and part of speech are space separated.
# A POSITION may be a combination of two individual positions, say R1
# and R2. Then the program learns features such as R1 is a determiner
# AND R2 is a noun. Such combination POSITIONS are to be specified
# as follows - A:R1-R2
# Here, A: stands for a combination feature (And). The positions 
# combined are hyphen separated.
#
# USAGE :  learnPOS.pl [OPTIONS] OUTPUT SOURCE POSITION [POSITION]...
#
# OPTIONS:
#     --mincount MIN	 the feature must occur atleast MIN times to 
#			 qualify. Default value is 0.
#     --help             Prints the help message.\n";
#     --version          Prints the version number.\n\n"; 
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("mincount=i", "version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }
# if version has been requested, show version
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}
# Capture the mincount
if ( defined $opt_mincount )
{
    $MIN = $opt_mincount;
    # CHECK for valid MIN
}
else {
    print "Assuming mincount to be 0\n";
    $MIN = 0;
}

# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 2 ) {
    print "Insufficient command line arguments.\n";
    &usage();
    exit;}

$opfl = shift();
$inpfl = shift();

# Capture the positions
foreach $position (@ARGV) {
	$FEATURES{$position}{exists} = 1;
}

if(-f $opfl) {  
    print "Destination file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y") {
	print "Overwriting the file.\n"; }
    else{
	exit; } }
 
# Open the various files
open(INP,"<$inpfl");
open(OP,">$opfl");

# Read lines from input file
while($ln = <INP>) {
    # Ignore blank lines
    if($ln =~ /^\s*$/){
	next;}
    
    foreach $Cposition (keys %FEATURES) {
	$nogo = 0;
	$compPOS = "UNDEFINED";
	
	# Take care of and and or features
	$operation = 0;
	if($Cposition =~ /^A:/)	{ $operation = 1; }
	if($Cposition =~ /^O:/)   { next; }
	
	# Split compond features
	if($operation > 0) {
	    $#indpositions = -1;
	    @indpositions = split(/-/, $Cposition);	
	    $indpositions[0] =~ s/^..//;
	}		
	else {
	    $#indpositions = -1;
	    $indpositions[0] = $Cposition;
	}
	
	foreach $position (@indpositions) {
	    
	    #############
	    if($position =~ /^[Ll]/) {
		# LEFT Feature
		# Extract N
		$N = "UNDEFINED";
		if($position =~ /^[lL](\d+)/) { $N = $1; }
		if($N eq "UNDEFINED") {
		    print"Invalid value for Left position: $position\n";
		    exit;
		}
		# Capture pos at the position
		$pos = "UNDEFINED";
		$skip = $N - 1;
		if($ln =~ /\s<p=\"([^\"]+)\"\/>\s((([^<]|<[^p]|<p[^=])[^\s]*\s)*<p=\"[^\"]+\"\/>\s){$skip}(([^<]|<[^p]|<p[^=])[^\s]*\s)*<head/) {
		    # print "FOUND \n";
		    $pos = $1;
		}
	    }
	    else {
		# RIGHT Feature
		# Extract N
		$N = "UNDEFINED";
		if($position =~ /^[rR](\d+)/) { $N = $1; }
		if($N eq "UNDEFINED") {
		    print"Invalid value for Right position: $position\n";
		    print "$ln\n";
		    exit;
		}
		# Capture pos at the position
		$pos = "UNDEFINED";
		$skip = $N - 1;
		if($ln =~ /<\/head>\s((([^<]|<[^p]|<p[^=])[^\s]*\s)*<p=\"[^\"]+\"\/>\s){$skip}(([^<]|<[^p]|<p[^=])[^\s]*\s)*<p=\"([^\"]+)\"\/>/) {
		    $tmppos = $&;
		    if($tmppos =~ /<p=\"([^\"]+)\"\/>$/) {
			$pos = $1;
		    }
		    else {
			print "Error in capturing pos: $tmppos\n";
			next;
		    }
		    if($pos =~ /~/) {
			print "$tmppos\n";
			next;
		    }
		}
	    }
	    
	    ###################
	    # Compound features
	    
	    if($pos eq "UNDEFINED") { $nogo = 1; last; }
	    if($compPOS eq "UNDEFINED") {
		$compPOS = $pos;
	    } else {
		$compPOS = $compPOS."-".$pos;
	    }
	}	
	
	if($nogo == 1)	{ next; }	
	$FEATURES{$Cposition}{$compPOS}++;
	
    } 
}

# CHECK IF count ok and print to file
foreach $position (sort keys %FEATURES) {
    foreach $pos (sort keys %{$FEATURES{$position}}) {
	if ($pos eq "exists")	{ next; }
	if ($pos eq "UNDEFINED") 	{ next; }
	if($FEATURES{$position}{$pos} > $MIN) {
	    print OP "$position $pos\n";
	}
    }
}

# OR Condition
# foreach $position (sort keys %FEATURES) {
 #   if($position !~ /O:/)	{ next; }
 #   $#indpositions = -1;
 #   @indpositions = split(/-/, $Cposition);
 #   $indpositions[0] =~ s/^..//;

 #   foreach $indpos (@indpositions) {
#	foreach $pos (sort keys %{FEATURES{$indpos}})
		
#        if($FEATURES{$position}{$pos} > $MIN) {
#            print OP "$position $pos\n";
#        }
#    }
#}


# Subroutine to print help message
sub help{
    print "Usage: learnPOS.pl [OPTIONS] OUTPUT SOURCE POSITION [POSITION]...\n\n"; 
    print "This program POS features from a given training file, SOURCE. The user specifies the positions of the features he is interested in, relative to the target word. L1 stands for the pos feature one position to the left of the target word. Ln  stands for the pos feature n positions to the left. Similary, R1, R2, ... , Rn. The program captures the pos occurring at the positions specified and the number of times they occur at that position.\n\n";
    print "OPTIONS:\n\n";
    print "  --mincount         any feature must must exist more than this\n";
    print "                     minimum number in SOURCE. Defaults to 0.\n";
    print "  --help             Prints the help message.\n";
    print "  --version          Prints the version number.\n\n"; }

sub usage{
    print "Usage: learnPOS.pl [OPTIONS] OUTPUT SOURCE POSITION [POSITION]...\n"; 
    AskHelp();}
# Message describing way to get help
sub AskHelp{
    print "Type learnPOS.pl --help for help.\n";}
# function to output the version number
sub showVersion
{
    print STDERR "learnPOS.pl      -        version 0.1\n";
    print STDERR "A component of SyntaLex version 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad Ted Pedersen\n";
}

