#!/usr/bin/perl -w

#### LAST UPDATED: Sep, 2003 ####
# parse2regex.pl version 0.1

###########################################################################
#
# This program creates regexes corresponding to the parse features listed 
# in the SOURCE file. Each line in the SOURCE file corresponds to one
# feature. Each feature entry consists of the broad category such as
# head word, parent phrase etc of the target word and a specific word 
# or phrase which is the head word or parent phrase and so on. 
# The regex created if matched with any line suggests that the line
# has the feature.
#
# USAGE :  parse2regex.pl [OPTIONS] OUTPUT SOURCE 
#
# OPTIONS:
#     --help             Prints the help message.
#     --version          Prints the version number.
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }
# if version has been requested, show version
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}

# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 1 ) {
    print "Insufficient command line arguments.\n";
    &usage();
    exit;}

$opfl = shift();
$inpfl = shift();

if(-f $opfl) {  
    print "Destination file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y") {
	print "Overwriting the file.\n"; }
    else{
	exit; } }

# Check if SOURCE exists
if(!(-f $inpfl)) {
    print "SOURCE file does not exist.\n";
    &usage();
    exit; }
 
# Open the various files
open(INP,"<$inpfl");
open(OP,">$opfl");

# Read lines from input file
while($ln = <INP>) {
	# Ignore blank lines
	if($ln =~ /^\s*$/){
	    next;}

	if($ln =~ /^\s*HD\s/) {
		# Feature: Head word
		# Capture head word
		$hdwd = "UNDEFINED";
		# Convert to lower case
		if($ln =~ /^\s*HD\s+([^\s]+)/) { $hdwd = $1; }
		if($hdwd eq "UNDEFINED") {
                        print"Invalid entry for head word: $ln\n";
                        exit;
                }

		# Escape the $ in PRP$, period, braces etc
                while($hdwd =~ /(^|[^\\])[\$\(\)\.\?\"]/) {
                        $hdwd =~ s/(^|[^\\])([\$\(\)\.\?\"])/$1\\$2/g;
                }

		# Print Regex
		$reg = "/\\s<P=\\\"[^~]+~$hdwd~[^~]+~[^\\\"]+\\\">\\s((([^<]|<[^P]|<P[^=])[^\\s]*\\s)*<P=\\\"[^~]+~[^~]+~[^~]+~[^\\\"]+\\\">\\s(([^<]|<[^P]|<P[^=])[^\\s]*\\s)*<\\/P>\\s(([^<]|<[^P]|<P[^=])[^\\s]*\\s)*)*(([^<]|<[^P\\\/]|<P[^=]|<\\\/[^P])[^\\s]*\\s)*<head/";
		print OP "$reg \@name = HDWD-$hdwd\n";
	} 

	# Parent
	if($ln =~ /^\s*PHD\s/) {
                # Feature: Head word
                # Capture head word
                $hdwd = "UNDEFINED";
                if($ln =~ /^\s*PHD\s+([^\s]+)/) { $hdwd = $1; }
                if($hdwd eq "UNDEFINED") {
                        print"Invalid entry for head word: $ln\n";
                        exit;
                }

                # Escape the $ in PRP$, period, braces etc
                while($hdwd =~ /(^|[^\\])[\$\(\)\.\?\"]/) {
                        $hdwd =~ s/(^|[^\\])([\$\(\)\.\?\"])/$1\\$2/g;
                }

                # Print Regex
                # Parent phrase head word = Parse tag + Multiple <P..>..</P> tags + Possible non <P and </P tags + Present phrase
                $reg = "/\\s<P=\\\"[^~]+~$hdwd~[^~]+~[^\\\"]+\\\">\\s((([^<]|<[^P]|<P[^=])[^\\s]*\\s)*<P=\\\"[^~]+~[^~]+~[^~]+~[^\\\"]+\\\">\\s(([^<]|<[^P]|<P[^=])[^\\s]*\\s)*<\\/P>\\s(([^<]|<[^P]|<P[^=])[^\\s]*\\s)*)*(([^<]|<[^P\\\/]|<P[^=]|<\\\/[^P])[^\\s]*\\s)*<P=\\\"[^~]+~[^~]+~[^~]+~[^\\\"]+\\\">\\s((([^<]|<[^P]|<P[^=])[^\\s]*\\s)*<P=\\\"[^~]+~[^~]+~[^~]+~[^\\\"]+\\\">\\s(([^<]|<[^P]|<P[^=])[^\\s]*\\s)*<\\/P>\\s(([^<]|<[^P]|<P[^=])[^\\s]*\\s)*)*(([^<]|<[^P\\\/]|<P[^=]|<\\\/[^P])[^\\s]*\\s)*<head/";

                print OP "$reg \@name = PHD-$hdwd\n";
        }

	# Phrase
	if($ln =~ /^\s*HDpos\s/) {
                # Feature: Head word pos
                # Capture head word's pos
                $hdpos = "UNDEFINED";
                if($ln =~ /^\s*HDpos\s+([^\s]+)/) { $hdpos = $1; }
                if($hdpos eq "UNDEFINED") {
                        print"Invalid entry for head word pos: $ln\n";
                        exit;
                }
                ## $hdwd = lc $hdwd;
                # Escape the $ in PRP$
                $hdpos =~ s/\$/\\\$/g;

                # Print Regex
                # Broad POS
                $reg = "/\\s<P=\\\"$hdpos([^~])*~[^~]+~[^~]+~[^\\\"]+\\\">\\s((([^<]|<[^P]|<P[^=])[^\\s]*\\s)*<P=\\\"[^~]+~[^~]+~[^~]+~[^\\\"]+\\\">\\s(([^<]|<[^P]|<P[^=])[^\\s]*\\s)*<\\/P>\\s(([^<]|<[^P]|<P[^=])[^\\s]*\\s)*)*(([^<]|<[^P\\\/]|<P[^=]|<\\\/[^P])[^\\s]*\\s)*<head/";

                print OP "$reg \@name = HDpos-$hdpos\n";
        }

	if($ln =~ /^\s*PHDpos\s/) {
                # Feature: Head word
                # Capture head word
                $hdwd = "UNDEFINED";
                if($ln =~ /^\s*PHDpos\s+([^\s]+)/) { $hdwd = $1; }
                if($hdwd eq "UNDEFINED") {
                        print"Invalid entry for head word: $ln\n";
                        exit;
                }

                # Escape the $ in PRP$, period, braces etc
                while($hdwd =~ /(^|[^\\])[\$\(\)\.\?\"]/) {
                        $hdwd =~ s/(^|[^\\])([\$\(\)\.\?\"])/$1\\$2/g;
                }

                # Print Regex
                # Parent phrase head word = Parse tag + Multiple <P..>..</P> tags + Possible non <P and </P tags + Present phrase
                $reg = "/\\s<P=\\\"$hdwd([^~])*~[^~]+~[^~]+~[^\\\"]+\\\">\\s((([^<]|<[^P]|<P[^=])[^\\s]*\\s)*<P=\\\"[^~]+~[^~]+~[^~]+~[^\\\"]+\\\">\\s(([^<]|<[^P]|<P[^=])[^\\s]*\\s)*<\\/P>\\s(([^<]|<[^P]|<P[^=])[^\\s]*\\s)*)*(([^<]|<[^P\\\/]|<P[^=]|<\\\/[^P])[^\\s]*\\s)*<P=\\\"[^~]+~[^~]+~[^~]+~[^\\\"]+\\\">\\s((([^<]|<[^P]|<P[^=])[^\\s]*\\s)*<P=\\\"[^~]+~[^~]+~[^~]+~[^\\\"]+\\\">\\s(([^<]|<[^P]|<P[^=])[^\\s]*\\s)*<\\/P>\\s(([^<]|<[^P]|<P[^=])[^\\s]*\\s)*)*(([^<]|<[^P\\\/]|<P[^=]|<\\\/[^P])[^\\s]*\\s)*<head/";

                print OP "$reg \@name = PHDpos-$hdwd\n";
        }

}

# Subroutine to print help message
sub help{
    print "Usage: parse2regex.pl [OPTIONS] OUTPUT SOURCE\n\n"; 
    print "This program creates regexes corresponding to the parse features listed in the SOURCE file. Each line in the SOURCE file corresponds to one feature. The regex created if matched with any line suggests that the line has the feature.\n\n";

    print "OPTIONS:\n\n";
    print "  --help             Prints the help message.\n";
    print "  --version          Prints the version number.\n\n"; }

sub usage{
    print "Usage: parse2regex.pl [OPTIONS] OUTPUT SOURCE\n"; 
    AskHelp();}
# Message describing way to get help
sub AskHelp{
    print "Type parse2regex.pl --help for help.\n";}
# function to output the version number
sub showVersion
{
    print STDERR "parse2regex.pl      -        version 0.1\n";
    print STDERR "A component of SyntaLex version 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad Ted Pedersen\n";
}

